@@ -2,8 +2,8 @@ package WWW::CheckSite;
 use strict;
 use warnings;
 
-# $Id: CheckSite.pm 682 2007-07-22 11:23:45Z abeltje $
-our $VERSION = '0.019_52';
+# $Id: CheckSite.pm 287 2005-04-03 14:26:15Z abeltje $
+our $VERSION = '0.013';
 
 =head1 NAME
 
@@ -47,14 +47,12 @@ site, the validator does not! Use this tool only on your own sites.
 
 =cut
 
-BEGIN { eval qq{ use Time::HiRes qw( time ); } }
+BEGIN { eval { use Time::HiRes qw( time ); } }
 use FindBin;
 use Storable qw( nstore retrieve );
 use File::Spec::Functions qw( :DEFAULT rel2abs );
 use File::Basename;
-use File::Path;
-use URI::file;
-
+use HTML::Template;
 use WWW::CheckSite::Validator;
 
 =head2 WWW::CheckSite->new( %args )
@@ -75,31 +73,6 @@ Initialize a new instance. Options supported:
 
 =item * B<validate> => by_none/by_uri/by_upload (by_none)
 
-=item * B<ua_class> => override the user agent class
-
-=item * B<ua_args> => hashref with extra options passed to the user agent class
-
-=item * B<v> => I<$verbosity>, where I<$verbosity> may be
-
-=item * B<tt> => boolean to allow the use of Template Toolkit
-
-=over
-
-=item 0
-
-Be quiet (default).
-
-=item 1
-
-Report basic information for every visited page (e.g. number of links
-and images) and total time for checking the site.
-
-=item 2
-
-Additional reporting of page validation details.
-
-=back
-
 =back
 
 =cut
@@ -108,7 +81,6 @@ sub new {
     my $class = shift;
 
     my %args = @_ ? ref $_[0] ? %{ +shift } : @_ : ();
-    exists $args{tt} or $args{tt} = 1;
 
     exists $args{uri} && length $args{uri} or
         _die( "", "Usage: WWW::CheckSite->new( uri => q<your_uri> )" );
@@ -116,9 +88,6 @@ sub new {
     exists $args{prefix} && length $args{prefix} or
         _die( "", "Usage: WWW::CheckSite->load( prefix => 'xxx' )" );
 
-    # Backward compatibility wrt {uri}
-    ref $args{uri} or $args{uri} = [ $args{uri} ];
-
     bless \%args, $class;
 }
 
@@ -132,8 +101,6 @@ Initialize the object from datafile. Supported options:
 
 =item * B<prefix> => the prefix used for this dataset [mandatory]
 
-=item * B<tt> => boolean to allow the use of Template Toolkit
-
 =back
 
 =cut
@@ -142,7 +109,6 @@ sub load {
     my $class = shift;
 
     my %args = ref $_[0] ? %{ +shift } : @_;
-    exists $args{tt} or $args{tt} = 1;
 
     exists $args{prefix} && length $args{prefix} or
         _die( "", "Usage: WWW::CheckSite->load( prefix => 'xxx' )" );
@@ -153,14 +119,7 @@ sub load {
     -f $wcsfile && -r _ or _die( "", "Cannot find '$wcsfile': $!" );
 
     my $self = retrieve $wcsfile;
-    $self->{v} and print "Loaded '$wcsfile'\n";
-    exists $args{ $_ } and $self->{ $_ } = $args{ $_ }
-        for qw( dir prefix tt v html_by css_by html_validator css_validator );
-
-    # Backward compatibility wrt {uri}
-    ref $self->{uri} or $self->{uri} = [ $self->{uri} ];
-
-    $self->_set_validator_fmt;
+    $self->{ $_ } = $args{ $_ } for qw( dir prefix );
 
     return $self;
 }
@@ -175,46 +134,35 @@ sub validate {
     my $self = shift;
 
     my $wcs = WWW::CheckSite::Validator->new(
-        uri            => $self->{uri},
-        ua_class       => $self->{ua_class},
-        ua_args        => $self->{ua_args},
-        exclude        => $self->{exclude},
-        html_validator => $self->{html_validator},
-        html_by        => $self->{html_by},
-        css_validator  => $self->{css_validator},
-        css_by         => $self->{css_by},
-        strictrules    => $self->{strictrules},
-        lang           => $self->{lang},
-        myrules        => $self->{myrules},
-        v               => $self->{v} > 1,
+        uri           => $self->{uri},
+        exclude_rules => $self->{exclude},
+        validate      => $self->{validate},
+        strict_rules  => $self->{strictrules},
+        lang          => $self->{lang},
+        v             => $self->{v} > 1,
     );
 
     my( $cnt, $intref ) = ( 0, 'a' );
     $self->{start_time} = time;
     while ( my $info = $wcs->get_page ) {
-        $info->{intref} = $intref++
-            if $info->{ret_uri} =~ /^\Q$self->{uri}[0]/;
-
+        $info->{intref} = $intref++ if $info->{ret_uri} =~ /^\Q$self->{uri}/;
         push @{ $self->{by_depth}{ $info->{depth} } }, $info->{ret_uri};
         $self->{report}{ $info->{ret_uri} } = $info;
 
         $self->{v} and printf "%5u %s (%u links; %u images; %s styles; %s)\n",
                               ++$cnt, @{ $info }{qw( ret_uri link_cnt 
                                                      image_cnt style_cnt)},
-                              $info->{valid} ? $info->{valid} != -1
-                                  ? 'valid' : 'skipped' : 'not valid';
+                              defined $info->{valid} ? $info->{valid} != -1
+                                  ? 'valid' : 'not valid' : 'skipped';
     }
     $self->{spider_time} = time;
 
     $self->{v} and printf "That took %s\n", $self->_spider_time;
 
-    $self->_set_validator_fmt;
-
     if ( $self->{save} ) {
         my $dir = $self->_datadir;
         unless ( -d $dir ) {
-            mkpath( $dir, $self->{v} ) or
-                $self->_die( "Cannot mkdir($dir): $!" );
+            mkdir $dir or $self->_die( "Cannot mkdir($dir): $!" );
         }
         nstore $self, $self->_datafile;
     }
@@ -222,46 +170,6 @@ sub validate {
 
 =begin private
 
-=head2 $wcs->_set_validator_fmt
-
-Set the base format for the validator uri in the reports. It is based
-on the W3 validator.
-
-=end private
-
-=cut
-
-sub _set_validator_fmt {
-    my( $self ) = @_;
-
-    my $val_base = $self->{html_validator}
-                || $WWW::CheckSite::Validator::VALIDATOR_XHTML;
-    $self->{validator_fmt} = "$val_base/check?uri=%s";
-}
- 
-=head2 $wcs->dump_links( $noskipped )
-
-Return a list with all URLs encountered during site-traversal.
-
-=cut
-
-sub dump_links {
-    my( $self, $noskipped ) = @_;
-
-    my %seen;
-    for my $url ( keys %{ $self->{report} } ) {
-        $seen{ $url }++;
-        for my $link ( @{ $self->{report}{ $url }{links} } ) {
-            $seen{ $link->{uri} }++
-                unless $noskipped && $link->{status} == 999;
-        }
-    }
-
-    return sort keys %seen;
-}
-
-=begin private
-
 =head2 $wcs->_spider_time
 
 Return time in hhmm()
@@ -330,39 +238,10 @@ Generate the reports.
 sub write_report {
     my $self = shift;
 
-    if ( $self->{tt} ) {
-        # first check if we have Template Toolkit
-        eval qq{use Template};
-        return $self->write_tt_report unless $@;
-    }
-
-    # then check for HTML::Template
-    eval qq{use HTML::Template};
-    return $self->write_ht_report unless $@;
-
-    $self->_die( "No supported template system found" );
-}
-
-=head2 $wcs->write_ht_report()
-
-Load, fill the L<HTML::Template> template and write the reports.
-
-=cut
-
-sub write_ht_report {
-    my $self = shift;
-
-    my $dir = $self->_datadir;
-    unless ( -d $dir ) {
-        mkpath( $dir, $self->{v} ) or $self->_die( "Cannot mkdir($dir): $!" );
-    }
-
-    my $mainuri = ref $self->{uri} ? $self->{uri}[0] : $self->{uri};
     for my $type (qw( summ full )) {
         $self->{rstart} = time;
-        my $report = create_report( "wcs${type}rpt.tmpl", $mainuri,
-                                    @{ $self }{qw( by_depth report
-                                                   validator_fmt v )} );
+        my $report = create_report( "wcs${type}rpt.tmpl",
+                                    @{ $self }{qw( uri by_depth report v )} );
         $self->{rfinish} = time;
 
         $report->param( 
@@ -377,66 +256,15 @@ sub write_ht_report {
         );
 
         # write the report
+        my $dir = $self->_datadir;
+        unless ( -d $dir ) {
+            mkdir $dir or $self->_die( "Cannot mkdir($dir): $!" );
+        }
         my $rptname = name_outfile( $self->_datadir, $type );
         open my $fh, "> $rptname" or
             $self->_die( "Cannot create($rptname): $!" );
         print $fh $report->output;
         close $fh or $self->_die( "Write error ($rptname): $!" );
-
-        my $furi = URI::file->new_abs( $rptname );
-        $self->{v} and print "Finished writing '$furi'\n";
-    }
-    return 1;
-}
-
-=head2 $wcs->write_tt_report()
-
-Load, fill the L<Template> Toolkit template and write the reports.
-
-=cut
-
-sub write_tt_report {
-    my $self = shift;
-
-    my $dir = $self->_datadir;
-    unless ( -d $dir ) {
-        mkpath( $dir, $self->{v} ) or
-            $self->_die( "Cannot mkdir($dir): $!" );
-    }
-
-    my $mainuri = ref $self->{uri} ? $self->{uri}[0] : $self->{uri};
-    my $data = create_report_data( 'all', $mainuri,
-                                   @{ $self }{qw( by_depth report
-                                                  validator_fmt v )} );
-    for my $type (qw( summ full )) {
-        my $tt_name = "wcs${type}rpt.tt";
-        $self->{rstart} = time;
-        my $report = Template->new({
-            INCLUDE_PATH => dirname( find_tmpl( $tt_name, $self->{v} ) ),
-            POST_CHOMP => 1,
-            EVAL_PERL  => 1,
-        });
-        $self->{rfinish} = time;
-
-        my $rptname = name_outfile( $self->_datadir, $type );
-        $report->process( $tt_name, {
-            ( map +( $_ => $self->{ $_ } ) => qw( uri by_depth report v ) ),
-            ( map +( $_ => $data->{ $_ } ) => keys %$data ),
-
-            spider_time  => $self->_spider_time,
-            report_time  => $self->_report_time,
-            now_time     => scalar localtime,
-            did_validate => ($self->{html_by} =~ /by_(?:upload|uri)/ ? 1 : 0),
-            strict_rules => $self->{strictrules},
-            language     => $self->{lang},
-            summlink     => basename( name_outfile( $self->_datadir, 'summ' ) ),
-            fulllink     => basename( name_outfile( $self->_datadir, 'full' ) ),
-        },
-            $rptname,
-        ) || $self->_die( $report->error );
-
-        my $furi = URI::file->new_abs( $rptname );
-        $self->{v} and print "Finished writing '$furi'\n";
     }
     return 1;
 }
@@ -453,43 +281,20 @@ sub _die {
     Carp::croak( @_ );
 }
 
+
 =head1 NO METHODS
 
 =head2 create_report()
 
-Load and fill the L<HTML::Template>.
-
 =cut
 
 sub create_report {
-    my( $tmplnm, $v ) = @_[ 0, -1 ];
-
-    my $data = create_report_data( @_ );
-
-    my $tmpl = HTML::Template->new(
-        filename => find_tmpl( $tmplnm, $v ),
-        loop_context_vars => 1,
-        die_on_bad_params => 0,
-    );
-    $tmpl->param( %$data );
-    return $tmpl;
-}
-
-=head2 create_report_data()
-
-Return a hash with all the data needed to fill both the
-L<HTML::Template> and the L<Template> Toolkit templates.
-
-=cut
-
-sub create_report_data {
-    my( $tmplnm, $url, $by_depth, $report, $validate_fmt, $v ) = @_;
+    my( $tmplnm, $url, $by_depth, $report, $v ) = @_;
 
     my %data = ( url => $url, title => $report->{ $url }{title},
                  valid_cnt => 0, valid_ok => 0, pages => [ ],
-                 not_ok_cnt => 0, kw_total => 0 );
+                 not_ok_cnt => 0 );
 
-    $v > 1 and print "Using validator: '$validate_fmt'\n";
     foreach my $level ( sort { $a <=> $b } keys %$by_depth ) {
         $v > 1 and printf "[$tmplnm]Level %u, %u page(s)\n", $level,
                           scalar @{ $by_depth->{ $level } };
@@ -497,17 +302,16 @@ sub create_report_data {
         foreach my $uri ( @{ $by_depth->{ $level } } ) {
             my $pinfo = $report->{ $uri };
             $pinfo->{uri} = $uri;
-            $pinfo->{validator_uri} = sprintf $validate_fmt, $uri;
             $pinfo->{status_tx} = status_text( $pinfo->{status} );
             $pinfo->{status_ok} = $pinfo->{status} == 200;
 
             $pinfo->{all_ok} = $pinfo->{status_ok};
-            for my $ikey (qw( links images styles )) {
-                $pinfo->{ "all_${ikey}_ok" } = 1;
-                $pinfo->{   "${ikey}_sk"   } = 0;
-                $pinfo->{ "kw_${ikey}_cnt" } = 0;
+            for my $all_ok (qw( links images styles )) {
+                $pinfo->{ "all_${all_ok}_ok"} = 1;
+            }
+            for my $skip (qw( links_sk images_sk styles_sk )) {
+                $pinfo->{ $skip } = 0;
             }
-
             foreach ( @{ $pinfo->{links} },
                       @{ $pinfo->{images} },
                       @{ $pinfo->{styles} } ) {
@@ -516,40 +320,33 @@ sub create_report_data {
                 $_->{status_ok} = $_->{status} == 200;
                 $_->{status_sk} = $_->{status} == 999;
                 exists $_->{valid} and
-                    $_->{status_ok} &&= $_->{valid};
-
+                    $_->{status_ok} &&= (defined $_->{valid}
+                                             ? $_->{valid} == 1 ? 1 : 0 : 1);
                 $pinfo->{all_ok} &&= ( $_->{status_ok} || $_->{status_sk} );
 
-                $_->{text} =~ s/>No text in TAG</&gt;No text in $_->{tag}&lt;/
+                $_->{text} =~ s/>No text in TAG</>No text in $_->{tag}</
                     and $_->{no_text} = 1;
-
                 $_->{"type_$_->{tag}"} = 1;
                 $pinfo->{all_ok} &&= ! $_->{no_text};
                 $_->{link_ok} = ! $_->{no_text} && $_->{status_ok};
                 if ( $_->{tag} eq 'link' ) { # this is a style
                     $_->{status_sk} and $pinfo->{styles_sk}++;
                     $pinfo->{all_styles_ok} &&= $_->{link_ok};
-                    $_->{valid_tx} = $_->{valid}
-                        ? $_->{valid} == 1 ? 'ok' : 'skipped' : 'not ok';
-                    $_->{valid_ok} = $_->{valid}
-                        ? $_->{valid} == 1 ? 1 : 1 : 0;
-                    $_->{link_ok} || $_->{status_sk}
-                        and $pinfo->{kw_styles_cnt}++;
+                    $_->{valid_tx} = defined $_->{valid}
+                        ? $_->{valid} == 1 ? 'ok' : 'not ok' : 'skipped';
+                    $_->{valid_ok} = defined $_->{valid}
+                        ? $_->{valid} == 1 ? 1 : 0 : 0;
                 } elsif ( exists $_->{ct} ) { # this is an image
                     $_->{status_sk} and $pinfo->{images_sk}++;
                     $pinfo->{all_images_ok} &&= $_->{link_ok};
-                    $_->{link_ok} || $_->{status_sk}
-                        and $pinfo->{kw_images_cnt}++;
                 } else {
                     $_->{status_sk} and $pinfo->{links_sk}++;
                     $pinfo->{all_links_ok} &&= $_->{link_ok};
-                    $_->{link_ok} || $_->{status_sk}
-                        and $pinfo->{kw_links_cnt}++;
                 }
             }
 
-            $pinfo->{valid_tx} = $pinfo->{valid}
-                ? $pinfo->{valid} == 1 ? 'ok' : 'skipped' : 'not ok';
+            $pinfo->{valid_tx} = defined $pinfo->{valid}
+                ? $pinfo->{valid} ? 'ok' : 'not ok' : 'skipped';
             defined $pinfo->{valid} and $pinfo->{all_ok} &&= $pinfo->{valid};
 
             $pinfo->{link_cnt} and $pinfo->{link_status_ok} =
@@ -559,14 +356,9 @@ sub create_report_data {
                     ? 'ok' : 'not ok' : 'N/A';
             $pinfo->{link_cnt} and 
                 $pinfo->{all_ok} &&= $pinfo->{link_status_ok};
-            $pinfo->{kw_links_cnt} ||= 0;
-            $pinfo->{kw_links} = $pinfo->{link_cnt}
-                ? sprintf "%.2f", $pinfo->{kw_links_cnt}/$pinfo->{link_cnt} : 1;
-            $pinfo->{status_ok} or $pinfo->{kw_links} = 0;
 
-            $pinfo->{kw_images_cnt} ||= 0;
-            $pinfo->{kw_images_cnt} and $pinfo->{image_status_ok} =
-                $pinfo->{kw_images_cnt} == $pinfo->{images_ok} +
+            $pinfo->{image_cnt} and $pinfo->{image_status_ok} =
+                $pinfo->{image_cnt} == $pinfo->{images_ok} +
                                        $pinfo->{images_sk}; 
             $pinfo->{image_status} = $pinfo->{image_cnt} 
                 ? $pinfo->{image_cnt} == $pinfo->{images_ok} +
@@ -574,9 +366,6 @@ sub create_report_data {
                     ? 'ok' : 'not ok' : 'N/A';
             $pinfo->{image_cnt} and 
                 $pinfo->{all_ok} &&= $pinfo->{image_status_ok};
-            $pinfo->{kw_images} = $pinfo->{image_cnt}
-                ? sprintf "%.2f", $pinfo->{kw_images_cnt}/$pinfo->{image_cnt}
-                : 1;
 
             $pinfo->{style_cnt} and $pinfo->{style_status_ok} =
                 $pinfo->{style_cnt} == $pinfo->{styles_ok} +
@@ -587,23 +376,6 @@ sub create_report_data {
                     ? 'ok' : 'not ok' : 'N/A';
             $pinfo->{style_cnt} and 
                 $pinfo->{all_ok} &&= $pinfo->{style_status_ok};
-            $pinfo->{kw_styles_cnt} ||= 0;
-            $pinfo->{kw_styles} = $pinfo->{style_cnt}
-                ? sprintf "%.2f", $pinfo->{kw_styles_cnt}/$pinfo->{style_cnt}
-                : 1;
-
-            $pinfo->{kw_return} = sprintf "%.2f", $pinfo->{status_ok} ? 1 : 0;
-            $pinfo->{kw_title}  = sprintf "%.2f", $pinfo->{title}
-                ? $pinfo->{status_ok} ? 1 : 0 : 0;
-            $pinfo->{kw_valid}  = sprintf "%.2f", 
-                                         $pinfo->{valid_tx} ne 'not ok' ? 1 : 0;
-            my @metrics = qw( return title valid links images styles );
-            $pinfo->{kw_total} = 0;
-            for my $metric (@metrics) {
-                $pinfo->{kw_total} += $pinfo->{ "kw_$metric" };
-            }
-            $pinfo->{kwalitee} = sprintf "%.2f", $pinfo->{kw_total}/@metrics;
-            $data{kw_site} += $pinfo->{kwalitee};
 
             $pinfo->{all_ok} or $data{not_ok_cnt}++;
             $pinfo->{valid} and $data{valid_ok}++;
@@ -612,12 +384,19 @@ sub create_report_data {
         }
     }
 
-    $data{page_cnt}    = scalar @{ $data{pages} };
-    $data{kwalitee}    = sprintf "%.2f", $data{kw_site}/$data{page_cnt};
-    $data{copyright}   = '&copy; MMV Abe Timmerman &lt;abeltje@cpan.org&gt;';
-    $data{wcs_version} = $VERSION;
+    $data{page_cnt} = scalar @{ $data{pages} };
 
-    return \%data;
+    my $tmpl = HTML::Template->new(
+        filename => find_tmpl( $tmplnm, $v ),
+        loop_context_vars => 1,
+        die_on_bad_params => 0,
+    );
+    $tmpl->param( %data,
+        copyright   => '&copy; MMV Abe Timmerman &lt;abeltje@cpan.org&gt;',
+        wcs_version => $VERSION,
+    );
+
+    return $tmpl;
 }
 
 =begin private
