@@ -2,46 +2,27 @@ package WWW::CheckSite::Spider;
 use strict;
 use warnings;
 
-# $Id: Spider.pm 681 2007-07-22 11:23:03Z abeltje $
+# $Id: Spider.pm 284 2005-04-02 11:12:40Z abeltje $
 use vars qw( $VERSION @EXPORT_OK %EXPORT_TAGS );
-$VERSION = '0.024';
+$VERSION = '0.006';
 
 =head1 NAME
 
-WWW::CheckSite::Spider - A base class for spidering the web
+WWW::CheckSite::Spider - Implement a base class for spidering
 
 =head1 SYNOPSIS
 
     use WWW::CheckSite::Spider;
 
     my $sp = WWW::CheckSite::Spider->new(
-         uri      => 'http://www.test-smoke.org',
+        ua_class => 'WWW::Mechanize', # or Win32:IE::Mechanize
+        uri      => 'http://www.test-smoke.org',
     );
 
     while ( my $page = $sp->get_page ) {
-        # $page is a hashref with basic information
+        # $page is a hashref with bais information
     }
 
-or to spider a site behind HTTP basic authentication:
-
-    package BA_Mech;
-    use base 'WWW::Mechanize';
-
-    sub get_basic_credentials { ( 'abeltje', '********' ) }
-
-    package main;
-    use WWW::CheckSite::Spider;
-
-    my $sp = WWW::CheckSite::Spider->new(
-         ua_class => 'BA_Mech',
-         uri      => 'http://your.site.with.ba/',
-    );
-
-    while ( my $page = $sp->get_page ) {
-        # $page is a hashref with basic information
-    }
-
-
 =head1 DESCRIPTION
 
 This module implements a basic web-spider, based on
@@ -49,12 +30,15 @@ C<WWW::Mechanize>. It takes care of putting pages on the
 "still-to-fetch" stack. Only uri's with the same origin will be
 stacked, taking the robots-rules on the server into account.
 
+=head1 METHODS
+
 =cut
 
 use WWW::CheckSite::Util;
 use WWW::RobotRules;
 use URI;
 
+# Define some constants for the cache mechanism
 =head1 CONSTATNTS & EXPORTS
 
 The following constants ar exported on demand with the B<:const> tag.
@@ -69,8 +53,6 @@ The following constants ar exported on demand with the B<:const> tag.
 
 =item B<WCS_TOSPIDER>
 
-=item B<WCS_TOFOLLOW>
-
 =item B<WCS_NOCONTENT>
 
 =item B<WCS_OUTSCOPE>
@@ -83,28 +65,25 @@ sub WCS_UNKNOWN()   {   0 }
 sub WCS_FOLLOWED()  {   1 }
 sub WCS_SPIDERED()  {   2 }
 sub WCS_TOSPIDER()  {   4 }
-sub WCS_TOFOLLOW()  {   8 }
 sub WCS_NOCONTENT() {  64 }
 sub WCS_OUTSCOPE()  { 128 }
 
-use base 'Exporter';
 %EXPORT_TAGS = (
-    const => [qw( WCS_UNKNOWN  WCS_FOLLOWED  WCS_SPIDERED WCS_TOSPIDER
-                  WCS_TOFOLLOW WCS_NOCONTENT WCS_OUTSCOPE )],
+    const => [qw( WCS_UNKNOWN  WCS_FOLLOWED  WCS_SPIDERED
+                  WCS_TOSPIDER WCS_NOCONTENT WCS_OUTSCOPE )],
 );
 @EXPORT_OK = map @{ $EXPORT_TAGS{ $_ } } => keys %EXPORT_TAGS;
-
-=head1 METHODS
+use base 'Exporter';
 
 =head2 WWW::CheckSite::Spider->new( %opts )
 
-Currently supported options (the rest will be set but not used!):
+Currently supported options (the rest will be passed!):
 
 =over 4
 
-=item * B<uri> => <start_uri> || <\@start_uri> [mandatory]
+=item * B<uri> => <start_uri> [mandatory]
 
-=item * B<ua_class> => by default L<WWW::Mechanize>
+=item * B<au_class> => 'WWW::Mechanize' or 'Win32::IE::Mechanize'
 
 =item * B<exclude> => <exclude_re> (qr/[#?].*$/)
 
@@ -116,7 +95,7 @@ Currently supported options (the rest will be set but not used!):
 
 =item * B<_self_base> => <my_base_to_use>
 
-=item * B<_norules> => perl_false 
+=item * B<_norules> => true false
 
 =end undocumented
 
@@ -130,23 +109,15 @@ sub new {
 
     $opts{uri} or do {
         require Carp;
-        Carp::croak( "No uri to spider specified!" );
+        Carp::croak( "No uri to spider specified!\n" );
     };
-    ref $opts{uri} or $opts{uri} = [ $opts{uri} ];
 
-    $opts{_self_base} ||= $opts{uri}->[0];
+    $opts{_self_base} ||= $opts{uri};
     $opts{_self_base} =~ s|^(.+/)(.+\.s?html?)|$1|;
-    $opts{_self_base} = URI->new( $opts{_self_base} )->canonical->as_string;
-
-    $opts{uri_base} = [ map {
-        ( my $base = $_ ) =~ s|^(.+/)(.+\.s?html?)|$1|;
-        URI->new( $base )->canonical->as_string;
-    } @{ $opts{uri} } ];
-    $opts{uri} = [map URI->new( $_ )->canonical->as_string => @{ $opts{uri} }];
 
     defined $opts{exclude} or $opts{exclude} = '[#?].*$';
     defined $opts{myrules} or $opts{myrules} = [ ];
-    $opts{_norules} = ! ( @{ $opts{myrules} } || $opts{strictrules} );
+    $opts{strictrules} and $opts{_norules} = 0;
 
     $opts{_stack} = new_stack();
     $opts{_cache} = new_cache();
@@ -156,133 +127,18 @@ sub new {
     my $self = bless \%opts, $class;
     $self->init_agent;
     $self->init_robotrules;
-    for ( my $i = $#{ $self->{uri} }; $i >= 0; $i-- ) {
-        my $uri  = $self->{uri}[ $i ];
-        my $base = $self->{uri_base}[ $i ];
-        if ( $self->uri_ok( $uri ) ) {
-            $self->{_stack}->push( [ $uri, $base ] );
-            $self->{_cache}->set( $uri => [ WCS_TOSPIDER, undef, 1 ] );
-        }
+    if ( $self->uri_ok( $self->{uri} ) ) {
+        $self->{_stack}->push( $self->{uri} );
+        $self->{_cache}->set( $self->{uri} => [ WCS_TOSPIDER, undef, 1 ] );
     }
 
     return $self;
 }
 
-=head2 $spider->get_page
+=head2 get_page
 
-Fetch the page and do some book keeping. It returns the result of
-C<< $pider->process_page() >>.
-
-=cut
-
-sub get_page {
-    my $self = shift;
-
-    my( $stack, $cache ) = @{ $self }{qw( _stack _cache )};
-    return unless $stack->size; # End of iteration
-
-    my $in_cache;
-    my $item = $stack->pop;
-    my $uri;
-    if ( ref $item ) {
-        $uri = $item->[0];
-        $self->{_self_base} = $item->[1];
-    } else {
-        $uri = $item;
-    }
-
-    $uri and $in_cache = $cache->has( $uri );
-    while ( defined $uri && $in_cache && !($in_cache->[0] & WCS_TOSPIDER) ) {
-        $uri = $stack->pop;
-        $uri and $in_cache = $cache->has( $uri );
-    }
-    return unless defined $uri; # End of iteration
-
-    $self->_process( $uri );
-}
-
-=begin private
-
-=head2 $self->_process( $uri )
-
-Private method to help not requesting pages more than once.
-
-=end private
-
-=cut
-
-sub _process {
-    my $self = shift;
-    my $uri  = shift;
-
-    my $mech = $self->current_agent;
-    $self->{v} and print "Fetch: '$uri': ";
-    $mech->get( $uri );
-    $self->{v} and printf "done(%sok).\n", $mech->success ? '' : 'not ';
-    $self->{_self_base} ||= $mech->base;
-
-    $self->_update_stack( $uri );
-
-    $self->process_page( $uri );
-}
-
-=begin private
-
-=head2 $self->_update_stack( $base )
-
-This is what the spider is all about. It will examine
-C<< $self->current_agent->links() >> to filter the links to follow.
-
-=end private
-
-=cut
-
-sub _update_stack {
-    my( $self, $base ) = @_;
-
-    my( $stack, $cache, $mech ) = @{ $self }{qw( _stack _cache _agent )};
-
-    my $this_page = $cache->has( $base );
-    @{ $this_page }[0, 1] = ( WCS_SPIDERED, $mech->status );
-
-    return unless $mech->success;
-
-    my @candidates = $self->links_filtered;
-
-    my $new_base = $mech->uri;
-
-    $new_base->canonical ne URI->new( $base )->canonical
-        and $cache->set( $new_base => $this_page );
-
-    foreach my $link ( @candidates ) {
-        my $new   = URI->new_abs( $link->url, $new_base )->as_string;
-        my $check = $self->strip_uri( $new );
-        my $data;
-        if ( $data = $cache->has( $check ) ) {
-            my $frag;
-            if ( $new ne $check && ! ($frag = $cache->has( $new )) ) {
-                $frag = [ WCS_TOFOLLOW, undef, $this_page->[2] + 1 ];
-                $cache->set( $new => $frag );
-            }
-        } else {
-            if ( $self->uri_ok( $new ) ) {
-                $stack->push( $check );
-                $data = [ WCS_TOSPIDER, undef, $this_page->[2] + 1 ];
-	    } else {
-                $data = [ WCS_OUTSCOPE, undef, $this_page->[2] + 1 ];
-                $cache->set( $new => $data );
-                $self->uri_ok( $check ) and 
-                    $data = [ WCS_TOFOLLOW, undef, $this_page->[2] + 1 ];
-            }
-            $cache->set( $check => $data );
-        }
-    }
-}
-
-=head2 $spider->process_page( $uri )
-
-Override this method to make the spider do something useful. By
-default it returns:
+Fetch the page and do some book keeping. It returns a hashref with
+some basic information:
 
 =over 4
 
@@ -306,10 +162,91 @@ default it returns:
 
 =cut
 
+sub get_page {
+    my $self = shift;
+
+    my( $stack, $cache ) = @{ $self }{qw( _stack _cache )};
+    return unless $stack->size; # End of iteration
+
+    my $in_cache;
+    my $uri = $stack->pop;
+    $uri and $in_cache = $cache->has( $uri );
+    while ( defined $uri && $in_cache && !($in_cache->[0] & WCS_TOSPIDER) ) {
+        $uri = $stack->pop;
+        $uri and $in_cache = $cache->has( $uri );
+    }
+    return unless defined $uri; # End of iteration
+
+    $self->_process( $uri );
+}
+
+=head2 _process
+
+Private method to help not requesting pages more than once.
+
+=cut
+
+sub _process {
+    my $self = shift;
+    my $uri  = shift;
+
+    my $mech = $self->{_agent};
+    $self->{v} and print "Fetch: '$uri': ";
+    $mech->get( $uri );
+    $self->{v} and printf "done(%sok).\n", $mech->success ? '' : 'not ';
+    $self->{_self_base} ||= $mech->base;
+
+    $self->update_stack( $uri );
+
+    $self->process_page( $uri );
+}
+
+=head2 update_stack
+
+This is what the spider is all about. It will examine
+C<< $self->{_agent}->links() >> to filter the links to follow.
+
+=cut
+
+sub update_stack {
+    my( $self, $base ) = @_;
+
+    my( $stack, $cache, $mech ) = @{ $self }{qw( _stack _cache _agent )};
+
+    my $this_page = $cache->has( $base );
+    @{ $this_page }[0, 1] = ( WCS_SPIDERED, $mech->status );
+
+    return unless $mech->success;
+
+    my @candidates = $self->links_filtered;
+
+    my $new_base = $mech->base;
+    foreach my $link ( @candidates ) {
+        my $check = URI->new_abs( $link->url, $new_base )->as_string;
+        my $data;
+        if ( $data = $cache->has( $check ) ) {
+        } else {
+            if ( $self->uri_ok( $check ) ) {
+                $stack->push( $check );
+                $data = [ WCS_TOSPIDER, undef, $this_page->[2] + 1 ];
+	    } else {
+                $data = [ WCS_OUTSCOPE, undef, $this_page->[2] + 1 ];
+            }
+            $cache->set( $check => $data );
+        }
+    }
+}
+
+=head2 process_page( $uri )
+
+Override this method to make the spider do something useful.
+
+=cut
+
 sub process_page {
     my( $self, $uri ) = @_;
 
-    my $mech = $self->current_agent;
+    my $mech = $self->{_agent};
 
     my $use_uri = $mech->success ? $mech->uri : $uri;
     my $in_cache = $self->{_cache}->has( $use_uri );
@@ -330,7 +267,7 @@ sub process_page {
     return $stats;
 }
 
-=head2 $spider->links_filtered
+=head2 links_filtered
 
 Filter out the uri's that will fail:
 
@@ -341,90 +278,25 @@ Filter out the uri's that will fail:
 sub links_filtered {
     my $self = shift;
     return grep {
-        my $url = $self->filter_link( $_->url );
-        ! defined $url
-            ? undef
-            : ( ($url eq $_->url and $_->url( $url )), $_ )
-    } $self->current_agent->links;
+        $_->url !~ m!^(?:mailto:|mms://|javascript:)!i
+    } $self->{_agent}->links;
 }
 
-=head2 $spider->filter_link( $uri )
+=head1 UserAgent
 
-Return the URI to be spidered or C<undef> for skipping.
-
-=cut
-
-sub filter_link {
-    my( $self, $uri ) = @_;
-    my $check = URI->new( $uri );
-    defined $check->scheme or return $uri;
-    return $check->scheme =~ /^(:?https|http|ftp|file)$/
-        ? $uri
-        : undef;
-}
-
-=head2 $spider->strip_uri( $uri )
-
-Strip the fragment bit of the I<$uri>.
-
-=cut
-
-sub strip_uri {
-    my $self = shift;
-
-    my $nu = URI->new( @_ );
-    defined $nu->fragment and $nu->fragment( undef );
-    return $nu->as_string
-}
-
-=head1 USERAGENT METHODS
-
-=head2 $spider->agent
+=head2 agent
 
 Retruns a standard name for this UserAgent.
 
 =cut
 
-sub agent { return (ref(shift) || __PACKAGE__) . "/$VERSION" }
+sub agent { return (ref( shift) || __PACKAGE__) . "/$VERSION" }
 
-=head2 $spider->init_agent
+=head2 init_agent
 
 Initialise the agent that is used to fetch pages. The default class is
 C<WWW::Mechanize> but any class that has the same methods will do.
 
-The C<ua_class> needs to support the following methods (see
-L<WWW::Mechanize> for more information about these):
-
-=over 4
-
-=item I<new>
-
-=item I<get>
-
-=item I<base>
-
-=item I<uri>
-
-=item I<status>
-
-=item I<success>
-
-=item I<ct>
-
-=item I<is_html>
-
-=item I<title>
-
-=item I<links>
-
-=item I<HEAD> (for L<WWW::CheckSite::Validator>)
-
-=item I<content> (for L<WWW::CheckSite::Validator>)
-
-=item I<images> (for L<WWW::CheckSite::Validator>)
-
-=back
-
 =cut
 
 sub init_agent {
@@ -432,15 +304,7 @@ sub init_agent {
     $self->{_agent} = $self->new_agent;
 }
 
-=head2 $spider->current_agent
-
-Return the current user agent.
-
-=cut
-
-sub current_agent { $_[0]->{_agent} }
-
-=head2 $spider->new_agent
+=head2 new_agent
 
 Create a new agent and return it.
 
@@ -450,12 +314,7 @@ sub new_agent {
     my $self = shift;
     $self->{ua_class} ||= 'WWW::Mechanize';
 
-    # If the package we're using has been declared inline, we don't
-    # don't want to try and use it...
-    # 20050421: by Pete Sergeant
-    unless ( exists $::{ $self->{ua_class} . '::' } ) { 
-        eval qq{ use $self->{ua_class} };
-    }
+    eval qq{ require $self->{ua_class}; };
     $@ and do {
         require Carp;
         Carp::croak( "Cannot initialise a UserAgent:\n$@" );
@@ -471,43 +330,17 @@ sub new_agent {
     return $ua;
 }
 
-=head1 ROBOTRULES METHODS
+=head1 Robot Rules
 
 The Spider uses the robot rules mechanism. This means that it will
-always get the F</robots.txt> file from the root of the webserver to
+always get the F<robots.txt> file from the root of the webserver to
 see if we are allowed (actually "not disallowed") to access pages as a
 robot.
 
 You can add rules for disallowing pages by specifying a list of lines
 in the F<robots.txt> syntax to C<< @{ $self->{myrules} } >>.
 
-=head2 $spider->more_rrules( $url )
-
-Check to see if the F<robots.txt> file for this C<$url> has already
-been loaded. If not, fetch the file and add the rules to the C<<
-$self->{_r_rules} >> object.
-
-=cut
-
-sub more_rrules {
-    my( $self, $url ) = @_;
-
-    my $rr_url = URI->new_abs( '/robots.txt', $url );
-
-    my $in_cache = $self->{_cache}->has( $rr_url );
-    unless ( $in_cache ) {
-        my $ua = $self->new_agent;
-        $self->{v} and print "Robot rules: $rr_url: ";
-        $ua->get( $rr_url );
-        $ua->success and $self->{_r_rules}->parse( $rr_url, $ua->content );
-        $self->{v} and printf "done(%sok).\n", $ua->success ? '' : 'not ';
-        $self->{v} && $ua->success
-            and printf "Setting rules:\n%s\n", $ua->content;
-        $self->{_cache}->set( $rr_url => [ WCS_SPIDERED, $ua->status, 0 ] );
-    }
-}
-
-=head2 $spider->uri_ok( $uri )
+=head2 uri_ok( $uri )
 
 This will determine whether this uri should be spidered. Rules are simple:
 
@@ -529,13 +362,12 @@ Is not excluded by F<robots.txt> mechanism
 sub uri_ok {
     my( $self, $uri ) = @_;
 
-    my $check_uri = URI->new( $uri );
     $self->{_uri_ok} = '';
     $self->{v} and print "  Check '$uri'";
-    $self->{_uri_ok} = 'scope'    unless $uri =~ m/^$self->{_self_base}/;
-    $self->{_uri_ok} = 'pattern'  if     $uri =~ m/$self->{exclude}/;
+    $self->{_uri_ok} = 'scope'   unless $uri =~ /^$self->{_self_base}/i;
+    $self->{_uri_ok} = 'pattern' if     $uri =~ m/$self->{exclude}/;
 
-    $self->{_uri_ok} = 'robots'   unless $self->{_norules} ||
+    $self->{_uri_ok} = 'robots'  unless $self->{_norules} ||
                                          $self->allowed( $uri );
 
     $self->{v} and
@@ -543,7 +375,7 @@ sub uri_ok {
     return !$self->{_uri_ok};
 }
 
-=head2 $spider->allowed( $uri )
+=head2 allowed( $uri )
 
 Checks the uri against the robotrules.
 
@@ -551,10 +383,10 @@ Checks the uri against the robotrules.
 
 sub allowed {
     my( $self, $uri ) = @_;
-    return $self->current_rrules->allowed( $uri );
+    $self->{_r_rules}->allowed( $uri );
 }
 
-=head2 $spider->init_robotrules( )
+=head2 init_robotrules( )
 
 This will setup a <WWW::RobotRules> object. C<< @{$self->{myrules } >>
 is used to add rules and should be in the RobotRules format. These
@@ -567,6 +399,7 @@ sub init_robotrules {
 
     my $agent = $self->agent;
     my $rules = WWW::RobotRules->new( $agent );
+    my $robot_agent = "User-agent: $agent\n";
 
     # The $base_url should be set!
     my $robots_uri = eval {
@@ -574,49 +407,26 @@ sub init_robotrules {
     };
     $@ and do {
         require Carp;
-        Carp::croak( "Error in base-url: $@" );
+        Carp::croak( "Error in base-url: $@\n" );
     };
+    $self->{v} and print "Robot rules: '$robots_uri': ";
 
-    my $robots_txt = "";
-    if ( $self->{strictrules} ) {
-        $self->{v} and print "Robot rules: '$robots_uri': ";
-        my $rua = $self->new_agent;
-        $rua->get( $robots_uri );
-        $self->{v} and printf "done(%sok).\n", $rua->success ? '' : 'not ';
-        $self->{_cache}->set( $robots_uri, [ WCS_SPIDERED, $rua->status, 0 ] );
-        $robots_txt = $rua->success ? $rua->content : "";
-    } else {
-        $self->{_cache}->set( $robots_uri, [ WCS_SPIDERED, 999, 0 ] );
-    }
+    my $rua = $self->new_agent;
+    $rua->get( $robots_uri );
+    $self->{v} and printf "done(%sok).\n", $rua->success ? '' : 'not ';
+    my $robots_txt = $rua->success ? $rua->content : $robot_agent;
+    $robots_txt ||= $robot_agent;
 
-    $robots_txt ||= @{ $self->{myrules} }
-        ? "User-Agent: *\n"
-        : "User-Agent: *\nDisallow:\n";
-
-    foreach my $myrule ( @{ $self->{myrules} } ) {
-        $robots_txt .= "Disallow: $myrule\n";
-        $self->{v} and print "myrule: Disallow: $myrule\n";
-    }
+    $robots_txt .= "Disallow: $_\n" foreach @{ $self->{myrules} };
 
     $robots_txt .= "Disallow:\n" if ( $robots_txt =~ tr/\n// ) == 1;
-    $self->{v} and print "Setting rules:\n$robots_txt\n";
 
     $rules->parse( $robots_uri, $robots_txt )
-        if $self->{uri}[0] =~ m|^https?://|; # problem with file:// uris
+        if $self->{uri} =~ m|^https?://|; # problem with file:// uris
 
     $self->{_r_rules} =  $rules;
 }
 
-=head2 $spider->current_rrules
-
-Returns the current RobotRules object.
-
-=cut
-
-sub current_rrules { $_[0]->{_r_rules} }
-
-1;
-
 =head1 AUTHOR
 
 Abe Timmerman, C<< <abeltje@cpan.org> >>
