@@ -2,13 +2,13 @@ package WWW::CheckSite::Validator;
 use strict;
 use warnings;
 
-# $Id: Validator.pm 672 2007-05-28 19:00:22Z abeltje $
-use vars qw( $VERSION $VALIDATOR_XHTML $VALIDATOR_STYLE $XMLLINT );
-$VERSION = '0.019';
+# $Id: Validator.pm 282 2005-04-02 09:48:07Z abeltje $
+use vars qw( $VERSION );
+$VERSION = '0.006';
 
 =head1 NAME
 
-WWW::CheckSite::Validator - A spider that assesses 'kwalitee' for a site
+WWW::CheckSite::Validator - Validate some of the html
 
 =head1 SYNOPSIS
 
@@ -17,7 +17,7 @@ WWW::CheckSite::Validator - A spider that assesses 'kwalitee' for a site
         uri => 'http://www.test-smoke.org'
     );
 
-    while ( my $info = $wcv->get_page ) {
+    while ( my $info = $wvc->get_page ) {
         # handle the info
     }
 
@@ -37,7 +37,7 @@ src> >>) are checked for availability.
 
 =item * B<images>
 
-All images on the page (C<< <img src> >>, C<< <input type=image> >>)
+All images on the page (C<< <img src> >>, C<< <imput type=image> >>)
 are checked for availability.
 
 =item * B<stylesheets>
@@ -58,44 +58,9 @@ validation.
 
 use WWW::CheckSite::Spider qw( :const );
 use base 'WWW::CheckSite::Spider';
-BEGIN {
-    $VALIDATOR_XHTML = 'http://localhost/w3c-validator/';
-    $XMLLINT         = 'xmllint';
-    $VALIDATOR_STYLE = 'http://localhost/css-validator/';
-}
-
-=head2 WWW::CheckSite::Validator->new( %args )
-
-Extend C<< WWW::CheckSite::Spider->new >> to check for L<Image::Info>
-so we can do a basic check on the images.
-
-On top of the attributes used by L<WWW::CheckSite::Spider>, this class
-uses:
-
-=over 4
-
-=item * B<html_by> => by_uri|by_upload|by_none
-
-=item * B<html_validator> => <uri>
-
-=item * B<css_by> => by_uri|by_upload|by_none
-
-=item * B<css_validator> => <uri>
-
-=back
-
-B<NOTE>: the I<validate> attrubute has been removed.
-
-=cut
-
-sub new {
-    my $class = shift;
-    my $self = $class->SUPER::new( @_ );
-
-    eval qq{use Image::Info qw( image_info )};
-    $self->{can_val_image} = ! $@;
-    return $self;
-}
+our $VALIDATOR_URL   = 'http://validator.w3.org/check?uri=%s';
+our $VALIDATOR_FRM   = 'http://validator.w3.org/';
+our $VALIDATOR_STYLE = 'http://jigsaw.w3.org/css-validator/';
 
 =head2 $wcs->process_page
 
@@ -152,9 +117,9 @@ page.  If there is no return status, it will C<HEAD> the uri and
 update the cache status for this link to prevent multiple HEADing.
 
 B<NOTE>: This method does B<not> respect the exclusion rules, and only
-robot-rules with C<strictrules> enabled!
+robot-rules with C<strict_rules> enabled!
 
-The structure for links:
+The structure for linkss:
 
 =over 4
 
@@ -182,15 +147,12 @@ sub check_links {
 
     my @links = $mech->success ? $self->links_filtered : ();
 
-    $self->{v} > 1 and printf "[check_links] found: %u\n", scalar @links;
     my @checked;
     for my $link ( @links ) {
-        my $check = URI->new_abs( $link->url, $mech->uri );
-        $self->more_rrules( $check );
+        my $check = URI->new_abs( $link->url, $mech->base );
         my $in_cache = $cache->has( $check );
         unless ( $in_cache && defined $in_cache->[1] ) {
-            if ( ! $self->allowed( $check ) ) {
-                $in_cache->[0] = WCS_OUTSCOPE;
+            if ( $self->{strict_rules} && !$self->allowed( $check ) ) {
                 $in_cache->[1] = '999';
                 $self->{v} and print "  HEAD '$check': skipped.\n";
             } else {
@@ -200,8 +162,8 @@ sub check_links {
                 $in_cache->[1] = $ua->status;
                 $self->{v} and
                     printf "done(%sok).\n", $ua->success ? '' : 'not ';
-
-                $ua->success && ! $self->ct_can_validate( $ua ) and
+    
+                $ua->success && $ua->ct ne 'text/html' and
                     $in_cache->[0] = WCS_NOCONTENT;
             }
 	}
@@ -225,7 +187,7 @@ sub check_links {
 
 =head2 $wcs->check_images( $stats )
 
-The C<check_images()> method gets information about the images on the
+The C<check_images()> method get information about the images on the
 page. The list comes from the I<images()> method of the mechanize
 object. It will only C<HEAD> the uri.
 
@@ -255,34 +217,21 @@ sub check_images {
 
     my @images = $mech->success ? $mech->images : ();;
 
-    $self->{v} > 1 and printf "[check_images] found: %u\n", scalar @images;
     my @checked;
     for my $img ( @images ) {
         my $check = URI->new_abs( $img->url, $mech->base );
-        $self->more_rrules( $check );
         my $in_cache = $cache->has( $check );
         defined $in_cache or
             $in_cache = $cache->set( $check => [ WCS_FOLLOWED ] );
         unless ( $in_cache && defined $in_cache->[1] ) {
-            $self->more_rrules( $check );
-            if ( ! $self->allowed( $check ) ) {
+            if ( $self->{strict_rules} && !$self->allowed( $check ) ) {
                 $in_cache->[1] = '999';
             } else {
+                $self->{v} and print "  HEAD '$check': ";
                 my $ua = $self->new_agent;
-                my $method = $self->{can_val_image} ? 'get' : 'head';
-                $self->{v} and print "  \U$method\E '$check': ";
-                eval { $ua->$method( $check ) };
-                my $success = $ua->success;
-
+                eval { $ua->head( $check ) };
                 $in_cache->[1] = $ua->status;
                 $in_cache->[2] = $ua->ct;
-                my $valid;
-                if ( $method eq 'head' ) {
-                    $valid = $success ? -1 : 0;
-                } else { # it's GET
-                    $valid = $success ? $self->validate_image( $ua ) : 0;
-		}
-                $in_cache->[3] = $valid;
                 $self->{v} and
                     printf "done(%sok).\n", $ua->success ? '' : 'not ';
             }
@@ -292,17 +241,14 @@ sub check_images {
             link   => $img->url,
             uri    => $check->as_string,
             tag    => 'ALT',
-            text   => ( defined( $img->alt )
-                ? ($img->alt || "")
-                : $self->{html_by} eq 'by_none' ? "" : ">No text in TAG<" ),
+            text   => $img->alt || ">No text in TAG<",
             status => $in_cache->[1],
             ct     => $in_cache->[2],
-            valid  => $in_cache->[3],
         };
     }
     $stats->{image_cnt} = @images;
     $stats->{images} = \@checked;
-    $stats->{images_ok} = grep $_->{status} == 200 && $_->{valid} => @checked;
+    $stats->{images_ok} = grep $_->{status} => @checked;
 
     return $stats;
 }
@@ -345,23 +291,19 @@ sub check_styles {
 	push @styles, $token->[1]{href};
     }
 
-    $self->{v} > 1 and printf "[check_styles] found: %u\n", scalar @styles;
     my @checked;
     for my $sheet ( @styles ) {
         my $check = URI->new_abs( $sheet, $mech->uri );
-        $self->more_rrules( $check );
         my $in_cache = $self->{_cache}->has( $check );
 
         defined $in_cache or
             $in_cache = $cache->set( $check => [ WCS_FOLLOWED ] );
         unless ( $in_cache && defined $in_cache->[1] ) {
-            $self->more_rrules( $check );
-            if ( ! $self->allowed( $check ) ) {
+            if ( $self->{strict_rules} && !$self->allowed( $check ) ) {
                 $in_cache->[1] = '999';
-                $in_cache->[3] = -1;
             } else {
                 my $ua = $self->new_agent;
-                my $method = $self->{css_by} =~ /by_(?:upload|uri)/
+                my $method = $self->{validate} =~ /by_(?:upload|uri)/
                     ? 'get' : 'head';
                 $self->{v} and print "  \U$method\E '$check': ";
                 eval { $ua->$method( $check ) };
@@ -370,8 +312,8 @@ sub check_styles {
                     printf "done(%sok).\n", $success ? '' : 'not ';
                 $in_cache->[1] = $ua->status;
                 $in_cache->[2] = $ua->ct;
-                $in_cache->[3] = $method eq 'get' && $success
-                                     ? $self->validate_style( $ua ) : -1;
+                $in_cache->[3] = $self->validate_style( $ua )
+                    if $method eq 'get' && $success;
             }
 	}
 
@@ -385,25 +327,7 @@ sub check_styles {
             valid  => $in_cache->[3],
         };
     }
-
-    my $inline_styles = $self->_extract_inline_styles;
-    $self->{v} > 1 and
-        printf "[validate_inline_style] found: %u\n", scalar @$inline_styles;
-    my $count = 1;
-    for my $style ( @$inline_styles ) {
-        $self->{v} > 1 and print "validate_inline_style: $count\n";
-        push @checked, {
-            link   => "InlineStyle-" . $count++,
-            uri    => '',
-            tag    => 'style',
-            text   => '',
-            status => 200,
-            ct     => 'text/css',
-            valid  => $self->validate_inline_style( $style ),
-        };
-    }
-
-    $stats->{style_cnt} = @checked;
+    $stats->{style_cnt} = @styles;
     $stats->{styles} = \@checked;
     $stats->{styles_ok}  = grep +($_->{status} == 200)  => @checked;
     $stats->{vstyles_ok} = grep
@@ -421,14 +345,13 @@ The C<validate()> method sends the url/contents off to W3.org to validate.
 sub validate {
     my( $self, $stats ) = @_;
 
-    unless ( $self->current_agent->success ) {
-        $self->{v} and
-            print "Validate @{[$self->current_agent->uri]}: skipped\n";
-        $stats->{valid} = -1;
+    unless ( $self->{_agent}->success ) {
+        $self->{v} and print "Validate @{[$self->{_agent}->uri]}: skipped\n";
+        $stats->{valid} = undef;
         return $stats;
     }
 
-    my $how_to = $self->{html_by} || 'by_none';
+    my $how_to = $self->{validate} || 'by_none';
     my $validate = "validate_$how_to";
     $self->can( $validate ) or $validate = 'validate_by_none';
 
@@ -443,7 +366,7 @@ The fallback do-not-validate method.
 
 sub validate_by_none {
     my( $self, $stats ) = @_;
-    $stats->{valid} = -1;
+    $stats->{valid} = undef;
 }
 
 =head2 $wcs->validate_by_uri
@@ -455,17 +378,15 @@ Sends only the uri to W3.ORG and get the validation result.
 sub validate_by_uri {
     my( $self, $stats ) = @_;
 
-    ( my $fmt = $self->{html_validator} ) =~ s!/+$!!;
-    $fmt .= '/check?uri=%s';
-    my $val_uri = sprintf $fmt, $self->current_agent->uri;
-    $self->{v} and print "HTML-Validate $val_uri: ";
+    my $val_uri = sprintf $VALIDATOR_URL, $self->{_agent}->uri;
+    $self->{v} and print "Validate @{[$self->{_agent}->uri]}: ";
 
     my $ua = $self->new_agent;
     $self->{lang} and $ua->default_header( 'Accept-Language' => 'en' );
     $ua->get( $val_uri );
 
     $stats->{valid} = $ua->success
-        ? $ua->content =~ /This Page Is Valid/ : '-1';
+        ? $ua->content =~ /This Page Is Valid/ ? '1' : '-1' : undef;
     $self->{v} and printf "done(%sok)\n", $stats->{valid} == 1 ? "" : "not ";
 
     $self->{lang} and $ua->default_header( 'Accept-Language' => $self->{lang} );
@@ -483,69 +404,31 @@ sub validate_by_upload {
     my( $self, $stats ) = @_;
 
     eval "use File::Temp";
-    $@ and $stats->{valid} = 1, return;
+    return if $@;
 
     my( $mech ) = @{ $self }{qw( _agent )};
     File::Temp->import( 'tempfile' );
     my( $fh, $filename ) = tempfile( 'wcvtempXXXX', SUFFIX => '.html', 
-                                                    UNLINK => 0 );
+                                                    UNLINK => 1 );
     print $fh $mech->content;
     close $fh;
 
-    $self->{v} and printf "HTML-Validate_upl(%s): %s ", $filename, $mech->uri;
+    $self->{v} and printf "Validate_upl(%s): %s ", $filename, $mech->uri;
     $stats->{validate} = $filename;
 
     my $ua = $self->new_agent;
     $self->{lang} and $ua->default_header( 'Accept-Language' => 'en' );
-    $ua->get( $self->{html_validator} );
-    if ( $ua->success ) {
-        $ua->submit_form( 
-            form_number => 2,
-            fields      => { uploaded_file => $filename },
-        );
-    }
+    $ua->get( $VALIDATOR_FRM );
+    $ua->submit_form( 
+        form_number => 2,
+        fields      => { uploaded_file => $filename },
+    );
 
     $stats->{valid} = $ua->success 
         ? $ua->content =~ /This Page Is Valid/ : -1;
     $self->{v} and printf " done(%sok)\n", $stats->{valid} == 1 ? "" : "not ";
 
     $self->{lang} and $ua->default_header( 'Accept-Language' => $self->{lang} );
-
-    -f $filename and unlink $filename;
-}
-
-=head2 $wcs->validate_by_xmllint( $stats )
-
-Use the L<xmllint(1)> program to validate the (X)HTML.
-
-=cut
-
-sub validate_by_xmllint {
-    my( $self, $stats ) = @_;
-    my $opts = qq[--postvalid --recover --stream];
-
-    eval "use File::Temp";
-    $@ and $stats->{valid} = 1, return;
-
-    my( $ua ) = @{ $self }{qw( _agent )};
-    File::Temp->import( 'tempfile' );
-    my( $fh, $filename ) = tempfile( 'wcvtempXXXX', SUFFIX => '.html', 
-                                                    UNLINK => 0 );
-    print $fh $ua->content;
-    close $fh;
-
-    my $cmd = qq[$self->{html_validator} $opts $filename 2>\&1];
-    $self->{v} and print "[$cmd]\n";
-    $self->{v} and printf "xmllint(%s): %s ", $filename, $ua->uri;
-    $stats->{validate} = $filename;
-
-    my $out = qx[$cmd];
-    $self->{v} and print $out;
-    $stats->{valid} = defined $out
-        ? $out eq '' : -1;
-    $self->{v} and printf " done(%sok)\n", $stats->{valid} == 1 ? "" : "not ";
-
-    -f $filename and unlink $filename;
 }
 
 =head2 $wcs->validate_style( $ua )
@@ -557,14 +440,10 @@ Dispatch the validation to the right method.
 sub validate_style {
     my( $self, $ua ) = @_;
 
-    $self->{css_validator} or return -1;
-    
-    my $how_to = $self->{css_by} || 'by_none';
+    my $how_to = $self->{validate} || 'by_none';
     my $validate = "style_$how_to";
     $self->can( $validate ) or $validate = 'style_by_none';
 
-    $self->{v} > 1 and print "[validate_style] $validate\n";
-
     $self->$validate( $ua );
 }
 
@@ -575,7 +454,7 @@ The fallback do-not-validate-stylesheet method.
 =cut
 
 sub style_by_none {
-    return -1;
+    return undef;
 }
 
 =head2 $wcs->style_by_uri( $ua )
@@ -588,17 +467,17 @@ sub style_by_uri {
     my( $self, $ua ) = @_;
 
     my $uri = $ua->uri;
-    $self->{v} and print "CSS-Validate $VALIDATOR_STYLE?$uri: ";
+    $self->{v} and print "CSS-Validate $uri: ";
     $self->{lang} and $ua->default_header( 'Accept-Language' => 'en' );
-    $ua->get( $self->{css_validator} );
+    $ua->get( $VALIDATOR_STYLE );
     $ua->submit_form(
         form_number => 1,
         fields      => { uri => $uri },
     );
 
     my $valid = $ua->success
-        ? $ua->content =~ /This document validates as / : -1;
-
+        ? $ua->content =~ /This document validates as /
+            ? '1' : '-1' : undef;
     $self->{v} and printf "done(%sok)\n", $valid == 1 ? "" : "not ";
 
     $self->{lang} and $ua->default_header( 'Accept-Language' => $self->{lang} );
@@ -616,119 +495,33 @@ call the validator with that temporary file and return the result.
 sub style_by_upload {
     my( $self, $ua ) = @_;
 
-    return $self->validate_upload_style( $ua, $ua->content );
-}
-
-=head2 $wcs->validate_inline_style( $style )
-
-Creates a new user-agent, and calls C<validate_upload_style()>.
-
-=cut
-
-sub validate_inline_style {
-    my( $self, $style ) = @_;
-
-    $self->{css_by} eq 'by_none' and return -1;
-    $self->{css_validator} or return -1;
-
-    my $ua = $self->new_agent;
-    return $self->validate_upload_style( $ua, $style );
-}
-
-=head2 $wcs->validate_upload_style( $ua, $style )
-
-Saves C<$style> to a temporary file and uploads it to the css-validator.
-
-=cut
-
-sub validate_upload_style {
-    my( $self, $ua, $style ) = @_;
-
     eval "use File::Temp";
     return if $@;
 
     File::Temp->import( 'tempfile' );
     my( $fh, $filename ) = tempfile( 'wcvtempXXXX', SUFFIX => '.css', 
-                                                    UNLINK => 0 );
-    print {$fh} $style;
+                                                    UNLINK => 1 );
+    print $fh $ua->content;
     close $fh;
 
-    my $uri = $ua->{req} ? eval { $ua->uri } : 'InlineStyle';
-    $self->{v} and printf "CSS-Validate_upl(%s): %s ", $filename, $uri;
+    $self->{v} and printf "CSS-Validate_upl(%s): %s ", $filename, $ua->uri;
 
     $self->{lang} and $ua->default_header( 'Accept-Language' => 'en' );
-    $ua->get( $self->{css_validator} );
-    if ( $ua->success ) {
-       $ua->submit_form( 
-           form_number => 2,
-           fields      => { file => $filename },
-        );
-    } else {
-        warn "[css_validator] " . $ua->status . "\n";
-    }
+    $ua->get( $VALIDATOR_STYLE );
+    $ua->submit_form( 
+        form_number => 2,
+        fields      => { file => $filename },
+    );
 
     my $valid = $ua->success 
-        ? $ua->content !~ m|<h2>Errors</h2>|i : -1;
+        ? $ua->content =~ /No error or warning found/ : -1;
     $self->{v} and printf " done(%sok)\n", $valid == 1 ? "" : "not ";
 
     $self->{lang} and $ua->default_header( 'Accept-Language' => $self->{lang} );
 
-    -f $filename and unlink $filename;
-
     return $valid;
 }
 
-=head2 $wcs->_extract_inline_styles
-
-Uses L<HTML::TokeParser> to extract inline styles from a document and
-returns a reference to an array with the contents of the inline style.
-
-=cut
-
-sub _extract_inline_styles {
-    my( $self ) = @_;
-    my $ua = $self->{_agent};
-    $ua->success or return [ ];
-
-    my @inline = ( );
-    my $p = HTML::TokeParser->new( \( $ua->content ) );
-    while ( my $t = $p->get_tag( 'style' ) ) {
-        ( my $style = $p->get_text( '/style' ) ) =~ s/\s+$//;
-        $style and push @inline, $style;
-        $self->{v} > 1 and print "[_extract_styles]$style\n";
-    }
-    return \@inline;
-}
-
-=head2 $wcs->validate_image( $ua )
-
-This is more like a basic consistency check, that uses C<<
-Image::Info::image_info() >>.
-
-=cut
-
-sub validate_image {
-    my( $self, $ua ) = @_;
-    my $image = $ua->content;
-    my $iinfo = Image::Info::image_info( \$image );
-    return ! $iinfo->{error};
-}
-
-=head2 $wcs->ct_can_validate( $ua )
-
-Check if the content-type is "validatable".
-
-=cut
-
-sub ct_can_validate {
-    my( $self, $ua ) = @_;
-
-    return $ua->ct =~ m[^\Qtext/html\E]                     ||
-           $ua->ct =~ m[^\Qtext/xhtml\E]                    ||
-           $ua->ct =~ m[^\Qapplication/xhtml+xml\E]         ||
-           $ua->ct =~ m[^\Qapplication/vnd.wap.xhtml+xml\E];
-}
-
 =head2 $wcs->set_action
 
 Why?
@@ -737,11 +530,6 @@ Why?
 
 sub set_action {
     my( $self, $check, $in_cache ) = @_;
-
-    defined $in_cache && defined $in_cache->[0] or do {
-        require Carp;
-        Carp::confess( "[INTERNAL ERROR]: $check" );
-    };
     
     my $reason = ($in_cache->[0] & WCS_OUTSCOPE)  ? $self->{_uri_ok} eq 'scope'
         ? 'Out of scope' : 'Excluded by pattern' : '';
